# This software has been approved for release by the U.S. Geological Survey (USGS). 
# Although the software has been subjected to rigorous review, the USGS reserves 
# the right to update the software as needed pursuant to further analysis and 
# review. No warranty, expressed or implied, is made by the USGS or the U.S. 
# Government as to the functionality of the software and related material nor 
# shall the fact of release constitute any such warranty. Furthermore, the 
# software is released on condition that neither the USGS nor the U.S. Government 
# shall be held liable for any damages resulting from its authorized or 
# unauthorized use.

# Supplemental material for:
# Cotterill et al., 2020
# Parsing the effects of demography, climate, and management on recurrent brucellosis outbreaks in elk
# gavin.cotterill@aggiemail.usu.edu

##' Muddy Creek, m1

rm(list=ls())
dev.off()

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(pomp2)
library(tidyverse)
library(gridExtra)

theme_set(theme_bw())
set.seed(594709947L)

load("./out/muddy_m1_swe_04Aug.RData")

gg.gavin.theme <- function(textsize = 40){
  theme(axis.line = element_line(colour = "black"),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        legend.title = element_text(size = textsize*0.5, face = "bold"),
        legend.text = element_text(size = textsize*0.5),
        legend.key = element_rect(color = "transparent", fill = "white"),
        legend.key.height = unit(3, "line"),
        legend.key.width = unit(3, "line"),
        legend.key.size = unit(1, "cm"),
        #plot.title = element_blank(),
        plot.title = element_text(size = textsize*0.7),
        axis.title.x = element_text(size = textsize*0.7, face = "bold",
                                    margin = margin(t = 40)),
        axis.title.y = element_text(size = textsize*.7, face = "bold",
                                    margin = margin(r = 40)),
        axis.text.x = element_text(size = 30*0.8), 
        axis.text.y = element_text(size = 30*0.8), 
        plot.margin = unit(c(1, 1, 0, 1.2), "cm"))
}

m1 <- pomp(dat, 
           time = "dyear",
           t0 = 2004,
           rprocess = discrete_time(rproc, delta.t = 1), 
           rinit = initlz,
           dmeasure = dmeas,
           rmeasure = rmeas,
           covar = covariate_table(covar, times = "cyear"),
           statenames = c("S", "I", "R1", "R2", "truesp", "N"),
           paramnames = c("beta", "bp","mu", "sigma", "rho","gamma", "theta","iota",
                          "S_0","I_0","R1_0","R2_0"),
           partrans = parameter_trans(log = log_params, logit = logit_params))

#=========================
# look at global results
#=========================

summary(results_global$loglik) 

library(plyr)
all <- ldply(list(guess = guesses, result = subset(results_global, loglik > max(loglik) - 50)))

pairs(~loglik + beta + bp + mu + sigma + rho + gamma + theta + iota,
      data = all, col = ifelse(all$.id == "guess", grey(0.5), "red"), pch = 16)

pairs(~loglik + S_0 + I_0 + R1_0 + R2_0,
      data = all, col = ifelse(all$.id == "guess", grey(0.5), "red"), pch = 16)

#=================================================================
# simulate again with the 'mle' from the global search 
#=================================================================

paramnames <- c("beta", "bp", "mu",  "sigma", "rho", "gamma", "theta", "iota",
                "S_0", "I_0", "R1_0", "R2_0")

results <- rbind(results_local, results_global)

(m1_mle <- results[which.max(results[,"loglik"]),][paramnames])

m1_mle_vals <- c(beta = m1_mle[,"beta"], 
                 bp = m1_mle[,"bp"],
                 mu = m1_mle[,"mu"],
                 sigma = m1_mle[,"sigma"], 
                 rho = m1_mle[,"rho"],
                 gamma = m1_mle[,"gamma"], 
                 theta = m1_mle[,"theta"],
                 iota = m1_mle[,"iota"],
                 S_0 = m1_mle[,"S_0"], 
                 I_0 = m1_mle[,"I_0"], 
                 R1_0 = m1_mle[,"R1_0"], 
                 R2_0 = m1_mle[,"R2_0"])

wils <- cbind(dat, tot = covar$tot[-1])
wils <- wils%>%
  select(-dpop)%>%
  mutate(lo = NA, hi = NA)

library(Hmisc)

for(i in 1:nrow(wils)){
  if(wils$pos[i] %in% NA) next
  tmp95 <- binconf(wils$pos[i], wils$tot[i], alpha = 0.1, method = "wilson")
  wils$lo[i] <- tmp95[2]
  wils$hi[i] <- tmp95[3]
}

disease_3 <- m1 %>% 
  simulate(params=m1_mle_vals,nsim=2e4,format="data.frame",include.data=TRUE)%>%
  subset(select=c(dyear,.id,apparent))%>%
  mutate(data=.id=="data") %>%
  ddply(~dyear+data,plyr::summarize,
        p=c(0.05,0.5,0.95),q=quantile(apparent,prob=p,names=FALSE, na.rm=T)) %>%
  mutate(p=mapvalues(p,from=c(0.05,0.5,0.95),to=c("lo","med","hi")),
         data=mapvalues(data,from=c(TRUE,FALSE),to=c("data","simulation"))) %>%
  reshape2::dcast(dyear+data~p,value.var='q') %>%
  ggplot()+
  geom_ribbon(aes(x=dyear,y=med,color=data,fill=data,ymin=lo,ymax=hi), alpha=0.2, show.legend = FALSE)+
  gg.gavin.theme()+
  xlab("")+
  ylab("Seroprevalence")+
  ggtitle("")+
  geom_errorbar(wils, mapping = aes(x = dyear, ymin = lo, ymax = hi, width = 0.2))+
  scale_x_discrete(limits = c(2005,2009, 2013, 2017))

pop_3 <- m1 %>% 
  simulate(params=m1_mle_vals,nsim=2e4,format="data.frame",include.data=TRUE)%>%
  subset(select=c(dyear,.id,dpop))%>%
  mutate(data=.id=="data") %>%
  ddply(~dyear+data,plyr::summarize,
        p=c(0.05,0.5,0.95),q=quantile(dpop,prob=p,names=FALSE, na.rm=T)) %>%
  mutate(p=mapvalues(p,from=c(0.05,0.5,0.95),to=c("lo","med","hi")),
         data=mapvalues(data,from=c(TRUE,FALSE),to=c("data","simulation"))) %>%
  reshape2::dcast(dyear+data~p,value.var='q')%>%
  ggplot()+
  geom_ribbon(aes(x=dyear,y=med,color=data,fill=data,ymin=lo,ymax=hi), alpha=0.2, show.legend = FALSE)+
  gg.gavin.theme()+
  xlab("")+
  ylab("Female count")+
  ggtitle("")+
  scale_x_discrete(limits = c(2005,2009, 2013, 2017))


grid.arrange(disease_3, pop_3, ncol = 2)

################
### FOI/SIRR ###
################

nsim = 2e3

sims3 <- simulate(m1, 
                  params = m1_mle_vals,
                  nsim = nsim, 
                  format = "data.frame", include.data = TRUE)

sims3 <- sims3[order(sims3$.id, sims3$dyear),]

MJ <- sims3[1:14, "MJ"]
sims3[15:nrow(sims3),"MJ"] <- rep(MJ, nsim)

beta_df <- sims3%>%  
  filter(.id != "data")%>%
  mutate(beta_foi = m1_mle[,"beta"] * (I + m1_mle[,"iota"]) / N^m1_mle[,"theta"])%>%
  ddply(~dyear,plyr::summarize,
        p=c(0.05,0.5,0.95),qBeta=quantile(beta_foi,prob=p,names=FALSE, na.rm=T)) %>%
  mutate(p=mapvalues(p,from=c(0.05,0.5,0.95),to=c("lo","med","hi"))) %>%
  reshape2::dcast(dyear~p,value.var='qBeta')

bp_df <- sims3%>%  
  filter(.id != "data")%>%
  mutate(bp_foi = m1_mle[,"bp"] * MJ * (I + m1_mle[,"iota"]) / N^m1_mle[,"theta"])%>%
  ddply(~dyear,plyr::summarize,
        p=c(0.05,0.5,0.95),qBp=quantile(bp_foi,prob=p,names=FALSE, na.rm=T)) %>%
  mutate(p=mapvalues(p,from=c(0.05,0.5,0.95),to=c("lo","med","hi"))) %>%
  reshape2::dcast(dyear~p,value.var='qBp')

muddy_foi <- ggplot()+
  geom_ribbon(data = beta_df, aes(x = dyear, y = med, ymin = lo, ymax = hi), 
              color = 'red', fill = "red", alpha = 0.2, show.legend = FALSE)+
  geom_ribbon(data = bp_df, aes(x = dyear, y = med, ymin = lo, ymax = hi), 
              color = 'blue', fill = "blue", alpha = 0.2, show.legend = FALSE)+
  gg.gavin.theme()+
  labs(x = "", y = "FOI at beta MLE")+
  ggtitle("muddy m1, 2k sims. at global MLE")+
  scale_x_discrete(limits = c(2005,2009, 2013, 2017))
muddy_foi


# SIRR

testS <- m1 %>% 
  simulate(params=m1_mle_vals,nsim=nsim,format="data.frame",include.data=TRUE)%>%
  subset(select=c(dyear,.id, S))%>%
  subset(.id!="data")%>%
  ddply(~dyear,plyr::summarize,
        p=c(0.05,0.5,0.95),qS=quantile(S,prob=p,names=FALSE, na.rm=T)) %>%
  mutate(p=mapvalues(p,from=c(0.05,0.5,0.95),to=c("lo","med","hi"))) %>%
  reshape2::dcast(dyear~p,value.var='qS')

testI <- m1 %>% 
  simulate(params=m1_mle_vals,nsim=nsim,format="data.frame",include.data=TRUE)%>%
  subset(select=c(dyear,.id, I))%>%
  subset(.id!="data")%>%
  ddply(~dyear,plyr::summarize,
        p=c(0.05,0.5,0.95),qI=quantile(I,prob=p,names=FALSE, na.rm=T)) %>%
  mutate(p=mapvalues(p,from=c(0.05,0.5,0.95),to=c("lo","med","hi"))) %>%
  reshape2::dcast(dyear~p,value.var='qI')

testR1 <- m1 %>% 
  simulate(params=m1_mle_vals,nsim=nsim,format="data.frame",include.data=TRUE)%>%
  subset(select=c(dyear,.id,R1))%>%
  subset(.id!="data")%>%
  ddply(~dyear,plyr::summarize,
        p=c(0.05,0.5,0.95),qR1=quantile(R1,prob=p,names=FALSE, na.rm=T)) %>%
  mutate(p=mapvalues(p,from=c(0.05,0.5,0.95),to=c("lo","med","hi"))) %>%
  reshape2::dcast(dyear~p,value.var='qR1')

testR2 <- m1 %>% 
  simulate(params=m1_mle_vals,nsim=nsim,format="data.frame",include.data=TRUE)%>%
  subset(select=c(dyear,.id, R2))%>%
  subset(.id!="data")%>%
  ddply(~dyear,plyr::summarize,
        p=c(0.05,0.5,0.95),qR2=quantile(R2,prob=p,names=FALSE, na.rm=T)) %>%
  mutate(p=mapvalues(p,from=c(0.05,0.5,0.95),to=c("lo","med","hi"))) %>%
  reshape2::dcast(dyear~p,value.var='qR2')


muddy_sirr <- ggplot()+
  geom_ribbon(data = testS, aes(x=dyear,y=med,ymin=lo,ymax=hi), 
              color = "green", fill = "green", alpha=0.2, show.legend = FALSE)+
  geom_ribbon(data = testI, aes(x=dyear,y=med,ymin=lo,ymax=hi), 
              color = "red", fill = "red", alpha=0.2, show.legend = FALSE)+
  geom_ribbon(data = testR1, aes(x=dyear,y=med,ymin=lo,ymax=hi), 
              color = "orange", fill = "orange", alpha=0.2, show.legend = FALSE)+
  geom_ribbon(data = testR2, aes(x=dyear,y=med,ymin=lo,ymax=hi), 
              color = "grey", fill = "grey", alpha=0.2, show.legend = FALSE)+
  gg.gavin.theme()+
  xlab("")+
  ylab("Individuals")+
  ggtitle("SIRR composition")+
  scale_x_discrete(limits = c(2005,2009, 2013, 2017))
muddy_sirr

grid.arrange(muddy_foi, muddy_sirr, ncol = 2)


# AIC calc
results[which(results$loglik == max(results$loglik)),]
(ll <- max(results$loglik))
# loglik and se columns aren't params, hence: -2
k <- length(results_global) - 2

(AIC_muddy_m1 <- -2 * ll + 2*k) 


