# This software has been approved for release by the U.S. Geological Survey (USGS). 
# Although the software has been subjected to rigorous review, the USGS reserves 
# the right to update the software as needed pursuant to further analysis and 
# review. No warranty, expressed or implied, is made by the USGS or the U.S. 
# Government as to the functionality of the software and related material nor 
# shall the fact of release constitute any such warranty. Furthermore, the 
# software is released on condition that neither the USGS nor the U.S. Government 
# shall be held liable for any damages resulting from its authorized or 
# unauthorized use.
#
# Supplemental material for:
# Cotterill et al., 2020
# Parsing the effects of demography, climate, and management on recurrent brucellosis outbreaks in elk
# gavin.cotterill@aggiemail.usu.edu

###################
### Muddy Creek ###
###################

rm(list = ls())

setwd("~/uncertain")
library(pomp2)
library(tidyverse)
set.seed(594709947L)

# Import endogenous results with sero+ TAS. This provides one model and basis for param. vectors

load("./muddy_m0_swe_04Aug.RData")

m0 <- pomp(dat, 
           time = "dyear",
           t0 = 2004,
           rprocess = discrete_time(rproc, delta.t = 1), 
           rinit = initlz,
           dmeasure = dmeas,
           rmeasure = rmeas,
           covar = covariate_table(covar, times = "cyear"),
           statenames = c("S", "I", "R1", "R2", "truesp", "N"),
           paramnames = c("beta","mu","sigma", "rho","gamma", "theta","iota",
                          "S_0","I_0","R1_0","R2_0"),
           partrans = parameter_trans(log = log_params, logit = logit_params))

# Build other models: no action, 10% sero- slaughter, 7.5% culling
# no-action 
rproc1 <- Csnippet("
                   double foi, muI, muR1;
                   double rate[9], trans[9];
                   
                   // rem I and R1
                   if(t == 2006) {
                   muI = 0;
                   muR1 = 0;
                   } else if (t == 2007) {
                   muI = 0;
                   muR1 = 0;
                   } else if (t == 2008) {
                   muI = 0;
                   muR1 = 0;
                   } else if (t == 2009) {
                   muI = 0;
                   muR1 = 0;
                   } else if (t == 2010) {
                   muI = 0;
                   muR1 = 0;
                   } else {
                   muI = 0;
                   muR1 = 0;
                   }; 
                   // expected force of infection
                   foi = beta * (I + iota) / pow(pop, theta);
                   
                   rate[0] = foi * dt;        // S to I/R1, force of infection
                   rate[1] = mu;              // S natural death
                   rate[2] = sigma;           // I to R1, rate of recovery
                   rate[3] = mu;              // I natural death
                   rate[4] = muI;             // I management removals      
                   rate[5] = gamma;           // R1 to R2, rate of detectable antibody loss
                   rate[6] = mu;              // R1 natural death
                   rate[7] = muR1;            // R1 management removals
                   rate[8] = mu;              // R2 natural death
                   
                   // transitions between classes
                   reulermultinom(2, S, &rate[0], dt, &trans[0]);
                   reulermultinom(3, I, &rate[2], dt, &trans[2]);
                   reulermultinom(3, R1, &rate[5], dt, &trans[5]);
                   reulermultinom(1, R2, &rate[8], dt, &trans[8]);
                   
                   S += births   - trans[0] - trans[1];
                   I += nearbyint(trans[0] * (1 - rho)) - trans[2] - trans[3] - trans[4];
                   R1 += nearbyint(trans[0] * rho) + trans[2] - trans[5] - trans[6] - trans[7];
                   R2 += trans[5] - trans[8]; 
                   N = S + I + R1 + R2;
                   truesp = (I + R1) / N;               // true seroprevalence
                   ")




m01 <- pomp(dat, 
            time = "dyear",
            t0 = 2004,
            rprocess = discrete_time(rproc1, delta.t = 1), 
            rinit = initlz,
            dmeasure = dmeas,
            rmeasure = rmeas,
            covar = covariate_table(covar, times = "cyear"),
            statenames = c("S", "I", "R1", "R2", "truesp", "N"),
            paramnames = c("beta","mu", "sigma", "rho","gamma","theta","iota", 
                           "S_0","I_0","R1_0","R2_0"),
            partrans = parameter_trans(log = log_params, logit = logit_params))


# 10% sero- removal:
rproc3 <- Csnippet("
                    double foi, muNeg;
                   double rate[9], trans[9];
                   
                   // rem S and R2
                   if(t >= 2006 && t <= 2010) {
                   muNeg = rem;
                   } else {
                   muNeg = 0;
                   }; 
                   // expected force of infection
                   foi = beta * (I + iota) / pow(pop, theta);
                   
                   rate[0] = foi * dt;        // S to I/R1, force of infection
                   rate[1] = mu;              // S natural death
                   rate[2] = muNeg;             // S and R2 management removals      
                   rate[3] = sigma;           // I to R1, rate of recovery
                   rate[4] = mu;              // I natural death
                   rate[5] = gamma;           // R1 to R2, rate of detectable antibody loss
                   rate[6] = mu;              // R1 natural death
                   rate[7] = mu;              // R2 natural death
                   rate[8] = muNeg;
                   
                   
                   // transitions between classes
                   reulermultinom(3, S, &rate[0], dt, &trans[0]);
                   reulermultinom(2, I, &rate[3], dt, &trans[3]);
                   reulermultinom(2, R1, &rate[5], dt, &trans[5]);
                   reulermultinom(2, R2, &rate[7], dt, &trans[7]);
                   
                   S += births   - trans[0] - trans[1]  - trans[2];
                   I += nearbyint(trans[0] * (1 - rho)) - trans[3] - trans[4];
                   R1 += nearbyint(trans[0] * rho) + trans[3] - trans[5] - trans[6];
                   R2 += trans[5] - trans[7]- trans[8]; 
                   N = S + I + R1 + R2;
                   truesp = (I + R1) / N;               // true seroprevalence
                   ")

# transform parameters for use with unconstrained optimizer
log_params3 <- c("beta", "iota")                                              # params > 0
logit_params3 <- c("mu", "rem", "sigma", "rho", "gamma", "theta", 
                   "S_0", "I_0", "R1_0", "R2_0")                              # 1 > params > 0                                  

m03 <- pomp(dat, 
            time = "dyear",
            t0 = 2004,
            rprocess = discrete_time(rproc3, delta.t = 1), 
            rinit = initlz,
            dmeasure = dmeas,
            rmeasure = rmeas,
            covar = covariate_table(covar, times = "cyear"),
            statenames = c("S", "I", "R1", "R2", "truesp", "N"),
            paramnames = c("beta","mu", "rem", "sigma", "rho","gamma","theta","iota", 
                           "S_0","I_0","R1_0","R2_0"),
            partrans = parameter_trans(log = log_params3, logit = logit_params3))

# 7.5% culling

rproc4 <- Csnippet("
                   double foi, muRem;
                   double rate[11], trans[11];
                   
                   // rem SIRR
                   if(t >= 2006 && t <= 2010) {
                   muRem = rem;
                   } else {
                   muRem = 0;
                   }; 
                   // expected force of infection
                   foi = beta * (I + iota) / pow(pop, theta);
                   
                   rate[0] = foi * dt;        // S to I/R1, force of infection
                   rate[1] = mu;              // S natural death
                   rate[2] = muRem;           // S management removals  
                   
                   rate[3] = sigma;           // I to R1, rate of recovery
                   rate[4] = mu;              // I natural death
                   rate[5] = muRem;           // I management removals   
                   
                   rate[6] = gamma;           // R1 to R2, rate of detectable antibody loss
                   rate[7] = mu;              // R1 natural death
                   rate[8] = muRem;           // R1 management removals   
                   
                   rate[9] = mu;              // R2 natural death
                   rate[10] = muRem;           // R2 management removals
                   
                   
                   // transitions between classes
                   reulermultinom(3, S, &rate[0], dt, &trans[0]);
                   reulermultinom(3, I, &rate[3], dt, &trans[3]);
                   reulermultinom(3, R1, &rate[6], dt, &trans[6]);
                   reulermultinom(2, R2, &rate[9], dt, &trans[9]);
                   
                   S += births - trans[0] - trans[1]  - trans[2];
                   I += nearbyint(trans[0] * (1 - rho)) - trans[3] - trans[4] - trans[5];
                   R1 += nearbyint(trans[0] * rho) + trans[3] - trans[6] - trans[7] - trans[8];
                   R2 += trans[6] - trans[9]- trans[10]; 
                   N = S + I + R1 + R2;
                   truesp = (I + R1) / N;               // true seroprevalence
                   ")

# redundant, but ok:
# transform parameters for use with unconstrained optimizer
log_params4 <- c("beta", "iota")                                              # params > 0
logit_params4 <- c("mu", "rem", "sigma", "rho", "gamma", "theta", 
                   "S_0", "I_0", "R1_0", "R2_0")                              # 1 > params > 0                                  

m04 <- pomp(dat, 
            time = "dyear",
            t0 = 2004,
            rprocess = discrete_time(rproc4, delta.t = 1), 
            rinit = initlz,
            dmeasure = dmeas,
            rmeasure = rmeas,
            covar = covariate_table(covar, times = "cyear"),
            statenames = c("S", "I", "R1", "R2", "truesp", "N"),
            paramnames = c("beta","mu", "rem", "sigma", "rho","gamma","theta","iota", 
                           "S_0","I_0","R1_0","R2_0"),
            partrans = parameter_trans(log = log_params4, logit = logit_params4))



##' set up 4 collections of 20 parameter vectors in a neighborhood of the maximum likelihood estimate
##' containing the region of high likelihood
results <- rbind(results_local, results_global) 

results %>%
  select(-loglik.se)%>%
  filter(loglik>max(loglik)-0.5*qchisq(df=1,p=0.99))%>%
  gather(parameter,value)%>%
  group_by(parameter)%>%
  dplyr::summarize(min=min(value),max=max(value))%>%
  ungroup()%>%
  filter(parameter!="loglik")%>%
  column_to_rownames("parameter")%>%
  as.matrix()->ranges

sobolDesign(lower=ranges[,"min"],
            upper=ranges[,"max"],
            nseq=20) -> params

##' Using these parameters as-is does not take into account the correlation of beta and theta
##' other parameters will also be correlated, eg S_0 and R1_0, but not nearly as strongly
##' we'll use the sobol design for all but beta+theta
##' for them, we'll again use the likelihood ratio test at 99%, plot the two parameters against
##' one another, and then characterize their distributions:

results %>%
  select(-loglik.se)%>%
  filter(loglik>max(loglik)-0.5*qchisq(df=1,p=0.99))%>%
  select(beta,theta)->bt

x <- bt$beta
y <- bt$theta
# plot(y~x)
fit <- lm(y ~ log(x))
# summary(fit)
x1=seq(from=min(x),to=max(x),length.out=1000)
y1=predict(fit,newdata=list(x=seq(min(x),to=max(x),length.out=1000)),interval="confidence")
# matlines(x1,y1,lwd=2)

# now generate 20 pairs to use
# plot(y~x)
x2=seq(from=min(x),to=max(x),length.out=20)
y2=predict(fit,newdata=list(x=seq(min(x),to=max(x),length.out=20)),interval="confidence")
# matlines(x2,y2,lwd=2)

params$beta <- x2
y2 <- as.data.frame(y2[,1])
names(y2)<-"theta"
y2[which(y2$theta < 0),] <- 0
y2[which(y2$theta > 1),] <- 1
params$theta <- y2[,1]

#' need 4 versions of 'params'
#' params = sero+ tas
#' params2 = no action
#' params3 = sero- 10%
#' params4 = cull 7.5%
params-> params2

params%>%
  mutate(rem = rep(0.10, 20)) -> params3

params%>%
  mutate(rem = rep(0.075, 20)) -> params4


#' Build foreach loops that: 
#'      - simulate all the models at the param vectors, 'params' varies according to model
#'      - save SIRR results for each model


library(foreach)
library(doParallel)
registerDoParallel(detectCores())
getDoParWorkers()
set.seed(998468235L, kind="L'Ecuyer")

### m0, sero+ removal

foreach(p=iter(params,by='row'), # select proper param vector
        .inorder=FALSE,
        .combine=bind_rows,
        .packages=c("tidyverse")
) %dopar% {
  
  library(pomp2)
  
  m0 %>% pfilter(params=p,Np=2000,save.states=TRUE) -> pf 
  
  pf@saved.states %>%               # latent state for each particle
    tail(1) %>%                     # last timepoint only
    melt() %>%                      # reshape and rename the state variables
    spread(variable,value) %>%
    group_by(rep) %>%
    dplyr::summarize(
      S_0=S,
      I_0=I,
      R1_0=R1,
      R2_0=R2
    ) %>%
    gather(variable,value,-rep) %>%
    spread(rep,value) %>%
    column_to_rownames("variable") %>%
    as.matrix() -> x
  ## the final states are now stored in 'x' as initial conditions
  
  ## set up a matrix of parameters
  pp <- parmat(unlist(p),ncol(x))
  
  ## generate simulations 
  m0 %>% 
    simulate(params=pp,format="data.frame",include.data=TRUE)%>%
    subset(select=c(dyear,.id, S, I , R1, R2))%>%
    mutate(
      data=.id=="data",
      loglik=logLik(pf)
    ) ->m0SIRR
  
  ## set the initial conditions to the final states computed above
  pp[rownames(x),] <- x
  
  merge(m0SIRR, m0SIRR)
} %>%
  mutate(weight=exp(loglik-mean(loglik))) %>%
  arrange(dyear,.id) -> sims.m0

## look at effective sample size
ess <- with(subset(sims.m0,dyear==max(dyear)),weight/sum(weight))
ess <- 1/sum(ess^2); ess

##' weighted quantile function
wquant <- function(x, weights, probs = c(0.05,0.5,0.95)){ # 90% quantile
  idx <- order(x)
  x <- x[idx]
  weights <- weights[idx]
  w <- cumsum(weights)/sum(weights)
  rval <- approx(w,x,probs,rule = 1)
  rval$y
}

sims.m0 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(S,weights=weight,probs=0.05),
    median=wquant(S,weights=weight,probs=0.5),
    upper=wquant(S,weights=weight,probs=0.95)
  ) -> simq.S.m0

sims.m0 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(I,weights=weight,probs=0.05),
    median=wquant(I,weights=weight,probs=0.5),
    upper=wquant(I,weights=weight,probs=0.95)
  ) -> simq.I.m0

sims.m0 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(R1,weights=weight,probs=0.05),
    median=wquant(R1,weights=weight,probs=0.5),
    upper=wquant(R1,weights=weight,probs=0.95)
  ) -> simq.R1.m0

sims.m0 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(R2,weights=weight,probs=0.05),
    median=wquant(R2,weights=weight,probs=0.5),
    upper=wquant(R2,weights=weight,probs=0.95)
  ) -> simq.R2.m0



### m02, no action

foreach(p=iter(params2,by='row'),
        .inorder=FALSE,
        .combine=bind_rows,
        .packages=c("tidyverse")
) %dopar% {
  
  library(pomp2)
  
  m01 %>% pfilter(params=p,Np=2000,save.states=TRUE) -> pf 
  
  pf@saved.states %>%               # latent state for each particle
    tail(1) %>%                     # last timepoint only
    melt() %>%                      # reshape and rename the state variables
    spread(variable,value) %>%
    group_by(rep) %>%
    dplyr::summarize(
      S_0=S,
      I_0=I,
      R1_0=R1,
      R2_0=R2
    ) %>%
    gather(variable,value,-rep) %>%
    spread(rep,value) %>%
    column_to_rownames("variable") %>%
    as.matrix() -> x
  ## the final states are now stored in 'x' as initial conditions
  
  ## set up a matrix of parameters
  pp <- parmat(unlist(p),ncol(x))
  
  ## generate simulations 
  m01 %>% 
    simulate(params=pp,format="data.frame",include.data=TRUE)%>%
    subset(select=c(dyear,.id, S, I , R1, R2))%>%
    mutate(
      data=.id=="data",
      loglik=logLik(pf)
    ) ->m02SIRR
  
  ## set the initial conditions to the final states computed above
  pp[rownames(x),] <- x
  
  merge(m02SIRR, m02SIRR)
} %>%
  mutate(weight=exp(loglik-mean(loglik))) %>%
  arrange(dyear,.id) -> sims.m02

## look at effective sample size
ess02 <- with(subset(sims.m02,dyear==max(dyear)),weight/sum(weight))
ess02 <- 1/sum(ess02^2); ess02

sims.m02 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(S,weights=weight,probs=0.05),
    median=wquant(S,weights=weight,probs=0.5),
    upper=wquant(S,weights=weight,probs=0.95)
  ) -> simq.S.m02

sims.m02 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(I,weights=weight,probs=0.05),
    median=wquant(I,weights=weight,probs=0.5),
    upper=wquant(I,weights=weight,probs=0.95)
  ) -> simq.I.m02

sims.m02 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(R1,weights=weight,probs=0.05),
    median=wquant(R1,weights=weight,probs=0.5),
    upper=wquant(R1,weights=weight,probs=0.95)
  ) -> simq.R1.m02

sims.m02 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(R2,weights=weight,probs=0.05),
    median=wquant(R2,weights=weight,probs=0.5),
    upper=wquant(R2,weights=weight,probs=0.95)
  ) -> simq.R2.m02



### m03, 10% sero- removal

foreach(p=iter(params3,by='row'),
        .inorder=FALSE,
        .combine=bind_rows,
        .packages=c("tidyverse")
) %dopar% {
  
  library(pomp2)
  
  m03 %>% pfilter(params=p,Np=2000,save.states=TRUE) -> pf 
  
  pf@saved.states %>%               # latent state for each particle
    tail(1) %>%                     # last timepoint only
    melt() %>%                      # reshape and rename the state variables
    spread(variable,value) %>%
    group_by(rep) %>%
    dplyr::summarize(
      S_0=S,
      I_0=I,
      R1_0=R1,
      R2_0=R2
    ) %>%
    gather(variable,value,-rep) %>%
    spread(rep,value) %>%
    column_to_rownames("variable") %>%
    as.matrix() -> x
  ## the final states are now stored in 'x' as initial conditions
  
  ## set up a matrix of parameters
  pp <- parmat(unlist(p),ncol(x))
  
  ## generate simulations 
  m03 %>% 
    simulate(params=pp,format="data.frame",include.data=TRUE)%>%
    subset(select=c(dyear,.id, S, I , R1, R2))%>%
    mutate(
      data=.id=="data",
      loglik=logLik(pf)
    ) ->m03SIRR
  
  ## set the initial conditions to the final states computed above
  pp[rownames(x),] <- x
  
  merge(m03SIRR, m03SIRR)
} %>%
  mutate(weight=exp(loglik-mean(loglik))) %>%
  arrange(dyear,.id) -> sims.m03

## look at effective sample size
ess03 <- with(subset(sims.m03,dyear==max(dyear)),weight/sum(weight))
ess03 <- 1/sum(ess03^2); ess03

sims.m03 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(S,weights=weight,probs=0.05),
    median=wquant(S,weights=weight,probs=0.5),
    upper=wquant(S,weights=weight,probs=0.95)
  ) -> simq.S.m03

sims.m03 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(I,weights=weight,probs=0.05),
    median=wquant(I,weights=weight,probs=0.5),
    upper=wquant(I,weights=weight,probs=0.95)
  ) -> simq.I.m03

sims.m03 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(R1,weights=weight,probs=0.05),
    median=wquant(R1,weights=weight,probs=0.5),
    upper=wquant(R1,weights=weight,probs=0.95)
  ) -> simq.R1.m03

sims.m03 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(R2,weights=weight,probs=0.05),
    median=wquant(R2,weights=weight,probs=0.5),
    upper=wquant(R2,weights=weight,probs=0.95)
  ) -> simq.R2.m03

### m04, 7.5% culling

foreach(p=iter(params4,by='row'),
        .inorder=FALSE,
        .combine=bind_rows,
        .packages=c("tidyverse")
) %dopar% {
  
  library(pomp2)
  
  m04 %>% pfilter(params=p,Np=2000,save.states=TRUE) -> pf 
  
  pf@saved.states %>%               # latent state for each particle
    tail(1) %>%                     # last timepoint only
    melt() %>%                      # reshape and rename the state variables
    spread(variable,value) %>%
    group_by(rep) %>%
    dplyr::summarize(
      S_0=S,
      I_0=I,
      R1_0=R1,
      R2_0=R2
    ) %>%
    gather(variable,value,-rep) %>%
    spread(rep,value) %>%
    column_to_rownames("variable") %>%
    as.matrix() -> x
  ## the final states are now stored in 'x' as initial conditions
  
  ## set up a matrix of parameters
  pp <- parmat(unlist(p),ncol(x))
  
  ## generate simulations 
  m04 %>% 
    simulate(params=pp,format="data.frame",include.data=TRUE)%>%
    subset(select=c(dyear,.id, S, I , R1, R2))%>%
    mutate(
      data=.id=="data",
      loglik=logLik(pf)
    ) ->m04SIRR
  
  ## set the initial conditions to the final states computed above
  pp[rownames(x),] <- x
  
  merge(m04SIRR, m04SIRR)
} %>%
  mutate(weight=exp(loglik-mean(loglik))) %>%
  arrange(dyear,.id) -> sims.m04

## look at effective sample size
ess04 <- with(subset(sims.m04,dyear==max(dyear)),weight/sum(weight))
ess04 <- 1/sum(ess04^2); ess04

sims.m04 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(S,weights=weight,probs=0.05),
    median=wquant(S,weights=weight,probs=0.5),
    upper=wquant(S,weights=weight,probs=0.95)
  ) -> simq.S.m04

sims.m04 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(I,weights=weight,probs=0.05),
    median=wquant(I,weights=weight,probs=0.5),
    upper=wquant(I,weights=weight,probs=0.95)
  ) -> simq.I.m04

sims.m04 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(R1,weights=weight,probs=0.05),
    median=wquant(R1,weights=weight,probs=0.5),
    upper=wquant(R1,weights=weight,probs=0.95)
  ) -> simq.R1.m04

sims.m04 %>%
  group_by(dyear) %>%
  summarize(
    lower=wquant(R2,weights=weight,probs=0.05),
    median=wquant(R2,weights=weight,probs=0.5),
    upper=wquant(R2,weights=weight,probs=0.95)
  ) -> simq.R2.m04


save.image("./out/muddy_m0_04Aug_unc_TAS_comparisons_04Aug19.RData")

