# This software has been approved for release by the U.S. Geological Survey (USGS). 
# Although the software has been subjected to rigorous review, the USGS reserves 
# the right to update the software as needed pursuant to further analysis and 
# review. No warranty, expressed or implied, is made by the USGS or the U.S. 
# Government as to the functionality of the software and related material nor 
# shall the fact of release constitute any such warranty. Furthermore, the 
# software is released on condition that neither the USGS nor the U.S. Government 
# shall be held liable for any damages resulting from its authorized or 
# unauthorized use.
#
# Supplemental material for:
# Cotterill et al., 2020
# Parsing the effects of demography, climate, and management on recurrent brucellosis outbreaks in elk
# gavin.cotterill@aggiemail.usu.edu

#' Interpretation: incorporating parameter uncertainty in Muddy Creek endogenous model results:
#' 1) seroprevalence and count trends
#' 2) FOI/SIRR
#' 3) TAS comparisons



###################
### Muddy Creek ###
###################

# Clear work space
rm(list=ls())
dev.off()
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(pomp2)
library(tidyverse)
library(gridExtra)

theme_set(theme_bw())
set.seed(594709947L)

load("./out/muddy_m0_04Aug_uncertain_04Aug19.RData")

gg.gavin.theme <- function(textsize = 40){
  theme(axis.line = element_line(colour = "black"),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        legend.title = element_text(size = textsize*0.5, face = "bold"),
        legend.text = element_text(size = textsize*0.5),
        legend.key = element_rect(color = "transparent", fill = "white"),
        legend.key.height = unit(3, "line"),
        legend.key.width = unit(3, "line"),
        legend.key.size = unit(1, "cm"),
        #plot.title = element_blank(),
        plot.title = element_text(size = textsize*0.7),
        axis.title.x = element_text(size = textsize*0.7, face = "bold",
                                    margin = margin(t = 40)),
        axis.title.y = element_text(size = textsize*.7, face = "bold",
                                    margin = margin(r = 40)),
        axis.text.x = element_text(size = 30*0.8), 
        axis.text.y = element_text(size = 30*0.8))
}


m0 <- pomp(dat, 
           time = "dyear",
           t0 = 2004,
           rprocess = discrete_time(rproc, delta.t = 1), 
           rinit = initlz,
           dmeasure = dmeas,
           rmeasure = rmeas,
           covar = covariate_table(covar, times = "cyear"),
           statenames = c("S", "I", "R1", "R2", "truesp", "N"),
           paramnames = c("beta","mu", "sigma", "rho","gamma", "theta","iota",
                          "S_0","I_0","R1_0","R2_0"),
           partrans = parameter_trans(log = log_params, logit = logit_params))

paramnames <- c("beta", "mu", "sigma", "rho", "gamma", "theta", "iota",
                "S_0", "I_0", "R1_0", "R2_0")

(m0_mle <- results[which.max(results[,"loglik"]),][paramnames])

m0_mle_vals <- c(beta = m0_mle[,"beta"], 
                 mu = m0_mle[,"mu"],
                 sigma = m0_mle[,"sigma"], 
                 rho = m0_mle[,"rho"],
                 gamma = m0_mle[,"gamma"], 
                 theta = m0_mle[,"theta"],
                 iota = m0_mle[,"iota"],
                 S_0 = m0_mle[,"S_0"], 
                 I_0 = m0_mle[,"I_0"], 
                 R1_0 = m0_mle[,"R1_0"], 
                 R2_0 = m0_mle[,"R2_0"])


wils <- cbind(dat, tot = covar$tot[-1])
wils <- wils%>%
  select(-dpop)%>%
  mutate(lo = NA, hi = NA)

library(Hmisc)

for(i in 1:nrow(wils)){
  if(wils$pos[i] %in% NA) next
  tmp90 <- binconf(wils$pos[i], wils$tot[i], alpha = 0.1, method = "wilson")
  wils$lo[i] <- tmp90[2]
  wils$hi[i] <- tmp90[3]
}

library(plyr)

disease_3 <- m0 %>% 
  simulate(params=m0_mle_vals,nsim=2e3,format="data.frame",include.data=TRUE)%>%
  subset(select=c(dyear,.id,apparent))%>%
  mutate(data=.id=="data") %>%
  ddply(~dyear+data,plyr::summarize,
        p=c(0.05,0.5,0.95),q=quantile(apparent,prob=p,names=FALSE, na.rm=T)) %>%
  mutate(p=mapvalues(p,from=c(0.05,0.5,0.95),to=c("lo","med","hi")),
         data=mapvalues(data,from=c(TRUE,FALSE),to=c("data","simulation"))) %>%
  reshape2::dcast(dyear+data~p,value.var='q') %>%
  ggplot()+
  geom_ribbon(aes(x=dyear,y=med,color=data,fill=data,ymin=lo,ymax=hi), alpha=0.2, show.legend = FALSE)+
  gg.gavin.theme()+
  theme(plot.margin = unit(c(1, 1, 0, 1.2), "cm"))+
  labs(x=NULL)+
  ylab("Seroprevalence")+
  ggtitle("Muddy Creek")+
  geom_errorbar(wils, mapping = aes(x = dyear, ymin = lo, ymax = hi, width = 0.2))+
  ylim(0,1)

simq.disease <- merge(simq.disease, wils)

dis_4 <- ggplot(data = simq.disease)+
  geom_line(aes(x = dyear, y = apparent), color = "red")+
  geom_errorbar(mapping = aes(x = dyear, ymin = lo, ymax = hi, width = 0.2))+
  geom_ribbon(aes(x=dyear,y=median,ymin=lower,ymax=upper), 
              alpha=0.2, show.legend = FALSE, color = "#00BFC4", fill = "#00BFC4")+
  gg.gavin.theme()+
  theme(plot.margin = unit(c(1, 1, 0, 1.2), "cm"))+
  labs(x=NULL)+
  ylab("Seroprevalence")+
  ggtitle("Muddy Creek")+
  ylim(0,1)

pop_3 <- m0 %>% 
  simulate(params=m0_mle_vals,nsim=2e3,format="data.frame",include.data=TRUE)%>%
  subset(select=c(dyear,.id,dpop))%>%
  mutate(data=.id=="data") %>%
  ddply(~dyear+data,plyr::summarize,
        p=c(0.05,0.5,0.95),q=quantile(dpop,prob=p,names=FALSE, na.rm=T)) %>%
  mutate(p=mapvalues(p,from=c(0.05,0.5,0.95),to=c("lo","med","hi")),
         data=mapvalues(data,from=c(TRUE,FALSE),to=c("data","simulation"))) %>%
  reshape2::dcast(dyear+data~p,value.var='q')%>%
  merge(.,data.frame(dyear=2004, data="data",hi=325,lo=325,med=325),all=T)%>%
  ggplot()+
  geom_ribbon(aes(x=dyear,y=med,color=data,fill=data,ymin=lo,ymax=hi), alpha=0.2, show.legend = FALSE)+
  gg.gavin.theme()+
  theme(plot.margin = unit(c(1, 1, 0, 1.2), "cm"))+
  labs(x=NULL)+
  ylab("Female count")+
  ggtitle("")+
  ylim(175,700)+
  scale_x_discrete(limits = c(2008, 2012, 2016))

simq.pop <- merge(simq.pop, dat)
add <- data.frame(dyear=2004, dpop=325)
simq.pop <- merge(simq.pop, add, all = T)

pop_4 <- ggplot(data = simq.pop)+
  geom_line(aes(x = dyear, y = dpop), color = "red")+
  geom_ribbon(aes(x=dyear,y=median,ymin=lower,ymax=upper), 
              alpha=0.2, show.legend = FALSE, color = "#00BFC4", fill = "#00BFC4")+
  gg.gavin.theme()+
  theme(plot.margin = unit(c(1, 1, 0, 1.2), "cm"))+
  labs(x=NULL)+
  ylab("Female count")+
  ggtitle("")+
  ylim(175,700)+
  scale_x_discrete(limits = c(2008, 2012, 2016))

grid.arrange(disease_3, pop_3, ncol = 2)


grid.arrange(dis_4, pop_4, ncol = 2)



########################
### Muddy Creek SIRR ###
########################


rm(list=ls())
dev.off()
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(pomp2)
library(tidyverse)
library(gridExtra)

theme_set(theme_bw())
set.seed(594709947L)

load("./out/muddy_m0_04Aug_unc_SIRR_04Aug19.RData")

gg.gavin.theme <- function(textsize = 40){
  theme(axis.line = element_line(colour = "black"),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        legend.title = element_text(size = textsize*0.5, face = "bold"),
        legend.text = element_text(size = textsize*0.5),
        legend.key = element_rect(color = "transparent", fill = "white"),
        legend.key.height = unit(3, "line"),
        legend.key.width = unit(3, "line"),
        legend.key.size = unit(1, "cm"),
        #plot.title = element_blank(),
        plot.title = element_text(size = textsize*0.7),
        axis.title.x = element_text(size = textsize*0.7, face = "bold",
                                    margin = margin(t = 40)),
        axis.title.y = element_text(size = textsize*.7, face = "bold",
                                    margin = margin(r = 40)),
        axis.text.x = element_text(size = 30*0.8), 
        axis.text.y = element_text(size = 30*0.8))
}


muddy_sirr <- ggplot()+
  geom_ribbon(data = simq.S, aes(x=dyear,y=median,ymin=lower,ymax=upper), 
              color = "green", fill = "green", alpha=0.2, show.legend = FALSE)+
  geom_ribbon(data = simq.I, aes(x=dyear,y=median,ymin=lower,ymax=upper), 
              color = "red", fill = "red", alpha=0.2, show.legend = FALSE)+
  geom_ribbon(data = simq.R1, aes(x=dyear,y=median,ymin=lower,ymax=upper), 
              color = "orange", fill = "orange", alpha=0.2, show.legend = FALSE)+
  geom_ribbon(data = simq.R2, aes(x=dyear,y=median,ymin=lower,ymax=upper), 
              color = "grey", fill = "grey", alpha=0.2, show.legend = FALSE)+
  gg.gavin.theme()+
  xlab("")+
  ylab("Individuals")+
  ggtitle("SIRR composition")+
  scale_x_discrete(limits = c(2005,2009, 2013, 2017))
muddy_sirr


######################################
### TAS comparisons, 50% quantiles ###
######################################

# Clear work space
rm(list=ls())
dev.off()

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(pomp2)
library(tidyverse)
library(gridExtra)

theme_set(theme_bw())
set.seed(594709947L)

load("./out/muddy_m0_04Aug_unc50_TAS_comparisons_04Aug19.RData")

gg.gavin.theme <- function(textsize = 40){
  theme(axis.line = element_line(colour = "black"),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        legend.title = element_text(size = textsize*0.5, face = "bold"),
        legend.text = element_text(size = textsize*0.5),
        legend.key = element_rect(color = "transparent", fill = "white"),
        legend.key.height = unit(3, "line"),
        legend.key.width = unit(3, "line"),
        legend.key.size = unit(1, "cm"),
        #plot.title = element_blank(),
        plot.title = element_text(size = textsize*0.7),
        axis.title.x = element_text(size = textsize*0.7, face = "bold",
                                    margin = margin(t = 40)),
        axis.title.y = element_text(size = textsize*.7, face = "bold",
                                    margin = margin(r = 40)),
        axis.text.x = element_text(size = 30*0.8), 
        axis.text.y = element_text(size = 30*0.8))
}

width=1.5

p1 <- ggplot()+
  geom_ribbon(data = simq.I.m0, aes(x=dyear,y=median,ymin=lower,ymax=upper), 
              color = "red", fill = "red", alpha=0.2, show.legend = FALSE)+
  geom_line(data = simq.I.m0, aes(x=dyear,y=median), size=width, linetype="dashed",
            color = "red", show.legend = FALSE)+
  
  geom_ribbon(data = simq.I.m02, aes(x=dyear,y=median,ymin=lower,ymax=upper),
              color = "grey", fill = "grey", alpha=0.2, show.legend = FALSE)+
  geom_line(data = simq.I.m02, aes(x=dyear,y=median), size=width, linetype="dashed", 
            color = "grey", show.legend = FALSE)+
  
  geom_ribbon(data = simq.I.m03, aes(x=dyear,y=median,ymin=lower,ymax=upper),
              color = "blue", fill = "blue", alpha=0.2, show.legend = FALSE)+
  geom_line(data = simq.I.m03, aes(x=dyear,y=median), size=width, linetype="dashed", 
            color = "blue", show.legend = FALSE)+
  
  geom_ribbon(data = simq.I.m04, aes(x=dyear,y=median,ymin=lower,ymax=upper),
              color = "orange", fill = "orange", alpha=0.2, show.legend = FALSE)+
  geom_line(data = simq.I.m04, aes(x=dyear,y=median), size=width, linetype="dashed", 
            color = "orange", show.legend = FALSE)+
  
  gg.gavin.theme()+
  theme(plot.margin = unit(c(1, 1, 0, 1.2), "cm"))+
  labs(x = NULL, y = "")+
  ggtitle("50% quantiles")+
  scale_x_discrete(limits = c(2008,2012, 2016))+
  geom_vline(xintercept= c(2006, 2010), linetype = "dashed")+
  ylim(0,40)

p1








