# This software has been approved for release by the U.S. Geological Survey (USGS). 
# Although the software has been subjected to rigorous review, the USGS reserves 
# the right to update the software as needed pursuant to further analysis and 
# review. No warranty, expressed or implied, is made by the USGS or the U.S. 
# Government as to the functionality of the software and related material nor 
# shall the fact of release constitute any such warranty. Furthermore, the 
# software is released on condition that neither the USGS nor the U.S. Government 
# shall be held liable for any damages resulting from its authorized or 
# unauthorized use.

# Supplemental material for:
# Cotterill et al., 2020
# Parsing the effects of demography, climate, and management on recurrent brucellosis outbreaks in elk
# gavin.cotterill@aggiemail.usu.edu

##' Greys River
##' m0 = internally-driven/endogenous model provided as example

library(pomp2)
library(tidyverse)

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

set.seed(594709947L)

dat <- read.csv("../../data/dat.csv", header = T)%>%
  filter(Feedground == "Greys River" & dyear >= 1993 & dyear <= 2017)

covar <- read.csv("../../data/covar.csv", header = T)%>%
  filter(Feedground == "Greys River" & cyear >= 1992 & cyear <= 2017)


#=============
# build model
#=============

rproc <- Csnippet("
                  double foi, mu;
                  double rate[7], trans[7];

                  // expected force of infection
                  foi = beta * (I + iota) / pow(pop, theta);

                  // variable death rate
                  if(t <= 2010) {
                    mu = mu1;
                  } else {
                    mu = mu2;
                  };

                  rate[0] = foi * dt;       // force of infection
                  rate[1] = mu;             // time-varying S death
                  rate[2] = sigma;          // rate of recovery
                  rate[3] = mu;             // time-varying I death
                  rate[4] = gamma;          // rate of detectable antibody loss
                  rate[5] = mu;             // time-varying R1 death
                  rate[6] = mu;            // time-varying R2 death

                  // transitions between classes
                  reulermultinom(2, S, &rate[0], dt, &trans[0]);
                  reulermultinom(2, I, &rate[2], dt, &trans[2]);
                  reulermultinom(2, R1, &rate[4], dt, &trans[4]);
                  reulermultinom(1, R2, &rate[6], dt, &trans[6]);

                  S += births   - trans[0] - trans[1];
                  I += nearbyint(trans[0] * (1 - rho)) - trans[2] - trans[3];
                  R1 += nearbyint(trans[0] * rho) + trans[2] - trans[4] - trans[5];
                  R2 += trans[4] - trans[6];
                  N = S + I + R1 + R2;
                  truesp = (I + R1) / N;               // true seroprevalence
                  ")

initlz <- Csnippet("
                   double m = pop / (S_0 + I_0 + R1_0 + R2_0);
                   S = nearbyint(m * S_0);
                   I = nearbyint(m * I_0);
                   R1 = nearbyint(m * R1_0);
                   R2 = nearbyint(m * R2_0);
                   truesp = (I + R1) / pop;
                   ")

dmeas <- Csnippet("if (ISNA(pos)) {
                    lik = (give_log) ? 0 : 1;
                  } else {
                    lik = dbinom(pos, tot, truesp, give_log) +
                      dnorm(dpop, N, 20, give_log); 
                  }")

rmeas <- Csnippet("dpop = rnorm(N, 20);
                  pos = rbinom(tot, truesp);
                  apparent = pos/tot;")

# transform parameters for use with unconstrained optimizer
log_params <- c("beta", "iota")                                # params > 0
logit_params <- c("mu1", "mu2", "sigma", "rho", "gamma", "theta",
                  "S_0", "I_0", "R1_0", "R2_0")                # 1 > params > 0


m0 <- pomp(dat,
           time = "dyear",
           t0 = 1992,
           rprocess = discrete_time(rproc, delta.t = 1),
           rinit = initlz,
           dmeasure = dmeas,
           rmeasure = rmeas,
           covar = covariate_table(covar, times = "cyear"),
           statenames = c("S", "I", "R1", "R2", "truesp", "N"),
           paramnames = c("beta","mu1", "mu2","sigma","rho","gamma","theta","iota",
                          "S_0","I_0","R1_0","R2_0"),
           partrans = parameter_trans(log = log_params, logit = logit_params))


paramvals <- c(beta = 0.05, # baseline transmission rate absent winter severity
               mu1 = 1/5, # death rate pre 2011
               mu2 = 1/9, # death rate post 2011
               sigma = .9, # rate of recovery I to R1
               rho = .5, # recovery S to R1
               gamma = .11, # rate of seroreversion
               theta = 0.1, # mixing parameter for degree of freq. v dens. dep.
               iota = 3, # annual imported infections
               S_0 = .3, I_0 = .1, R1_0 = .2, R2_0 = .4)

#=============================
# start sequential Monte Carlo  
#=============================

library(foreach)
library(doParallel)
registerDoParallel(detectCores())
getDoParWorkers()
set.seed(998468235L, kind="L'Ecuyer")


# a local search of the likelihood surface

stew(file = "./out/greys_m0.rda",{
  t_local_mif <- system.time({
    mifs_local <- foreach(i=1:20,
                          .packages='pomp2',
                          .combine=c,
                          .options.multicore=list(set.seed = TRUE),
                          .export=c("m0", "paramvals")
    ) %dopar%
    {
      mif2(
        m0,
        params=paramvals,
        Np=2e4,
        Nmif=800,
        cooling.type="geometric",
        cooling.fraction.50=0.25,
        rw.sd=rw.sd(beta=0.02,
                    mu1=ifelse(time <= 2010,0.02,0),
                    mu2=ifelse(time >= 2011,0.02,0),
                    sigma=0.02,
                    rho=0,
                    gamma=0,
                    theta=0.02,
                    iota=0.02,
                    S_0=ivp(0.02, lag = 1), 
                    I_0=ivp(0.02, lag = 1),
                    R1_0=ivp(0.02, lag = 1), 
                    R2_0=ivp(0.02, lag = 1))
      )
    }
  })
}, seed = 482947940, kind = "L'Ecuyer")
 

##' evaluate local log likelihood
stew(file = "./out/greys_m0_lik_local.rda", {
  t_local_eval <- system.time({
    results_local <- foreach(mf = mifs_local,
                             .packages = "pomp2",
                             .combine = rbind,
                             .options.multicore = list(set.seed = TRUE)
    ) %dopar%
    {
      evals <- replicate(10, logLik(pfilter(mf, Np = 2e4)))
      ll <- logmeanexp(evals, se = TRUE)
      c(coef(mf), loglik = ll[1], loglik = ll[2])
    }
  })
}, seed = 900242057, kind = "L'Ecuyer")


results_local <- as.data.frame(results_local)


#==================================================
# build a database to keep track of what we've done
#==================================================

#' use first time to keep the header:
write.csv(results_local, file = "./out/greys_m0_params.csv", row.names = FALSE)


#=========================
# start global search
#=========================

params_box <- rbind(
  beta = c(0, 10),
  mu1 = c(1/9, 1/3),
  mu2 = c(1/9, 1/3),
  sigma = c(0, 1),
  rho = c(0.49, 0.51),
  gamma = c(0.10, 0.12),
  theta = c(0, 1),
  iota = c(0, 5)
)

## ivp_guesses 
N <- 4e3

S <- runif(N, .3, .8) # S is constrained between .3 and .8, serop is ~20% at t0 
remainderS <- 1 - S

R1 <- rep(NA, N) # R1 is constrained between .1 and .4/ceiling
for(i in 1:N){
  if(remainderS[i] >= 0.4){
    R1[i] <- runif(1, .1, .4)
  }else if(remainderS[i] < .4 & remainderS[i] >= .1){
    R1[i] <- runif(1, .1, remainderS[i])
  }
}
remainderR1 <- 1 - S - R1

I <- rep(NA, N) # I is constrained between ~zero and .3/ceiling
for(i in 1:N){
  if(remainderR1[i] >= .3){
    I[i] <- runif(1, .0001, .3)
  }else if(remainderR1[i] < .3 & remainderR1[i] >= .0001){
    I[i] <- runif(1, .0001, remainderR1[i])
  }
}

R2 <- 1 - S - R1 - I 
df <- data.frame(S_0 = S, I_0 = I, R1_0 = R1, R2_0 = R2)
df <- df[-which(df$R2_0 > .2),] # R2 is constrained between ~zero, ceiling/0.2

ivp_guesses <- df[1:100,] # keep 100 guesses

rm(S, I, R1, R2, df, remainderS, remainderR1) # clean up

stew(file = "./out/greys_m0_global.rda", {
  n_global <- getDoParWorkers()
  t_global <- system.time({
    mf1 <- mifs_local[[1]]
    guesses <- as.data.frame(apply(params_box, 1, function(x)runif(100, x[1], x[2])))
    guesses <- cbind(guesses, ivp_guesses)
    results_global <- foreach(guess = iter(guesses, "row"),
                              .packages = "pomp2",
                              .combine = rbind,
                              .options.multicore = list(set.seed = TRUE)
    ) %dopar%
    {
      mf <- mif2(mf1, 
                 params=unlist(guess)
                 )
      mf <- mif2(mf, Nmif = 800)
      ll <- replicate(10, logLik(pfilter(mf, Np = 2e4)))
      ll <- logmeanexp(ll, se = TRUE)
      c(coef(mf), loglik = ll[1], loglik = ll[2])
    }
  })
}, seed = 1270401374, kind = "L'Ecuyer")



results_global <- as.data.frame(results_global)

# append results to database:
write.table(results_global, "./out/greys_m0_params.csv", sep = ",", col.names = FALSE,
            row.names = FALSE, append = TRUE)


save.image("./out/greys_m0.RData")


















